<?php

namespace Teknisa\COC\Controllers;

use Teknisa\GER\Utilities\Environment;
use Teknisa\GER\Services\CrudService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\Remote\FileUploader\Server;

class Coc00022ManutencaoMenugrid extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $parameterBagColumns = array();
	protected $dataSourceName;
	protected $dataSourceNameToSave;

	protected $fileUploader;
	protected $service;
	protected $crudService;
	protected $workflowService;
	protected $dataSourceManager;
	protected $connection;

	protected $fileserverPath;
	protected $pathKey;
	protected $undefinedImagePath;

	public function __construct($crudService, $parameterBag, $fileUploader, $fileServerParams, $connection, $dataSourceManager, $workflowService, $manutencaoMenugridService) {
		$this->crudService = $crudService;
		$this->fileUploader = $fileUploader;
		$this->connection = $connection;
		$this->dataSourceManager = $dataSourceManager;
		$this->workflowService = $workflowService;
		$this->service = $manutencaoMenugridService;

		$this->fileserverPath = $fileServerParams->getFileserverPath();
		$this->pathKey = $fileServerParams->getPathKey();
		$this->undefinedImagePath = $fileServerParams->getUndefinedImagePath();
		// $this->adService = $adService;

		parent::__construct($crudService, $parameterBag);
	}

	public function deleteLastProporcao($request, $response){
		$row = (array)$request->getRow();
		$row['NRORG'] = $this->crudService->getEnvironment()->getNrOrg();
		try{
			$this->connection->beginTransaction();
			$this->service->deleteLastProporcao($row);
			$this->connection->commit();
		} catch(\Exception $e){
			$response->setError(new Error($e->getMessage(),500));
			$this->connection->rollback();
		}
	}

	public function saveCicloInstrucaoMontagem($request, $response){
		$row = $request->getDataSet()->getRows()[0];
		try {
			$this->connection->beginTransaction();

			if($row['__is_new']){
				
				$row['FGFOTOSUBSEMAM'] = NULL;
				$row['FGFOTOSUBSEMAMT'] = NULL;

				if(isset($row['PHOTO_UPLOAD'][0]))
					$row['FGFOTOSUBSEMAMT'] = $row['PHOTO_UPLOAD'][0]['path'];

				if(isset($row['PHOTO_UPLOAD_THUMBNAIL'][0]))
					$row['FGFOTOSUBSEMAM'] = $row['PHOTO_UPLOAD_THUMBNAIL'][0]['path'];

			} else {
				
				if(isset($row['PHOTO_UPLOAD'][0])){
					
					if($row['OLD_FGFOTOSUBSEMAM'] != NULL)
						$this->service->tryDeleteImage($row);

					$row['FGFOTOSUBSEMAMT'] = $row['PHOTO_UPLOAD'][0]['path'];
					
				}
				
				if(isset($row['PHOTO_UPLOAD_THUMBNAIL'][0])){
					
					if($row['OLD_FGFOTOSUBSEMAM'] != NULL)
						$this->service->tryDeleteImage($row);

					$row['FGFOTOSUBSEMAM'] = $row['PHOTO_UPLOAD_THUMBNAIL'][0]['path'];
					
				}

				if (!isset($row['FGFOTOSUBSEMAM']) || $row['FGFOTOSUBSEMAM'] == '') {
						
					if(isset($row['OLD_FGFOTOSUBSEMAM']) && $row['OLD_FGFOTOSUBSEMAM'] != NULL)
						$this->service->tryDeleteImage($row);
					
					$row['FGFOTOSUBSEMAM'] = NULL;
					$row['FGFOTOSUBSEMAMT'] = NULL;
					
				} else {
					
					$row['FGFOTOSUBSEMAM'] = str_replace($this->fileserverPath, '', $row['FGFOTOSUBSEMAM']);
	
				}

			}


			$rowsCiclo = $this->getRowsWithDefaultDataToSave('auxciclosubgrusemam', array($row));
			$this->crudService->save('auxciclosubgrusemam', $rowsCiclo);
			$rowInstrucaoMontagem = $row;

			if(!$rowInstrucaoMontagem['__is_new']){
				if($rowInstrucaoMontagem['POSSUI_MONTAGEM'] == 'N' && $rowInstrucaoMontagem['DSMONTASUBSEMAMHTML']){
					$rowInstrucaoMontagem['__is_new'] = true;
					$rowsInstrucaoMontagem = $this->getRowsWithDefaultDataToSave('auxmontasubsemam',array($rowInstrucaoMontagem));
					$this->crudService->save('auxmontasubsemam', $rowsInstrucaoMontagem);
				} else if($rowInstrucaoMontagem['POSSUI_MONTAGEM'] == 'S' && $rowInstrucaoMontagem['DSMONTASUBSEMAMHTML']){
					$rowInstrucaoMontagem['__is_new'] = false;
					$rowsInstrucaoMontagem = $this->getRowsWithDefaultDataToSave('auxmontasubsemam',array($rowInstrucaoMontagem));
					$this->crudService->save('auxmontasubsemam', $rowsInstrucaoMontagem);
				}
			} else {
				if($rowInstrucaoMontagem['DSMONTASUBSEMAMHTML']) {
					$rowInstrucaoMontagem['__is_new'] = true;
					$rowsInstrucaoMontagem = $this->getRowsWithDefaultDataToSave('auxmontasubsemam',array($rowInstrucaoMontagem));
					$this->crudService->save('auxmontasubsemam', $rowsInstrucaoMontagem);
				}
			}
			$this->connection->commit();
		} catch (\Exception $e){
			$response->setError(new Error($e->getMessage(), 500));
			$this->connection->rollback();
		}
	}

	private function getRowsWithDefaultDataToSave($dataSourceName, $rows){
		$request = new DTO\Request\DataSet(new DataSet($dataSourceName, $rows), '','','');
		return $this->crudService->getRowsToSave($request, $dataSourceName);
	}

	public function findCicloInstrucaoMontagem($request, $response){
		$this->dataSourceName = 'auxciclosubgrusemam_ciclo';
		$parameterBagColumns = array('FILE_SERVER');
		$oldFilterCriteria = $request->getFilterCriteria();
		$conditions = $oldFilterCriteria->getConditions();
		$newConditions = array(
			'columnName' => 'FILE_SERVER',
			'operator' => '=',
			'value' => $this->fileserverPath
		);
		array_push($conditions, $newConditions);
		$newFilterCriteria = new FilterCriteria('auxciclosubgrusemam_ciclo', $conditions, $oldFilterCriteria->getPage(), $oldFilterCriteria->getPageSize());
		$newRequest = new DTO\Request\Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
		parent::find($newRequest, $response);
	}

	public function deleteSubgrupo($request, $response){
		$row =(array)$request->getRow();
		$this->dataSourceName = 'auxsubgrusemam';
		try{
			$this->connection->beginTransaction();
			$params = $this->service->getParamsToExecuteOperations($row);
			$ciclos = $this->getCiclos($params);
			$this->service->deleteSonDataSubgrupo($params);
			$this->service->deleteImagesCiclos($ciclos, NULL);
			parent::delete($request, $response);
			$this->connection->commit();
		} catch (\Exception $e){
			$response->setError(new Error($e->getMessage(), 500));
			$this->connection->rollback();
		}
	}

	private function getCiclos($row){
		$conditions = array(
			array(
				'columnName' => 'CDFILIAL',
				'operator' => '=',
				'value' => $row['CDFILIAL']
			),
			array(
				'columnName' => 'CDCIA',
				'operator' => '=',
				'value' => $row['CDCIA']
			),
			array(
				'columnName' => 'CDMANCIA',
				'operator' => '=',
				'value' => $row['CDMANCIA']
			),
			array(
				'columnName' => 'CDCLASSE',
				'operator' => '=',
				'value' => $row['CDCLASSE']
			),
			array(
				'columnName' => 'NRSERVICOCIAFILI',
				'operator' => '=',
				'value' => $row['NRSERVICOCIAFILI']
			),
			array(
				'columnName' => 'CDSUBGRU',
				'operator' => '=',
				'value' => $row['CDSUBGRU']
			),
			array(
				'columnName' => 'NRSEQSUBGRUSEMAM',
				'operator' => '=',
				'value' => $row['NRSEQSUBGRUSEMAM']
			)
		);
		$ciclos = $this->crudService->getFilteredDataSet('auxciclosubgrusemam', $conditions)->getRows();
		return $ciclos;
	}

	public function deleteCicloAndChildData($request, $response){
		$row = (array)$request->getRow();
		try {
			$this->dataSourceName = 'auxciclosubgrusemam';
			$this->connection->beginTransaction();
			//instrução de montagem - AUXMONTASUBSEMAM
			$this->service->executeOperationCiclo('AUXMONTASUBSEMAM',$row);

			//receitas e produtos - AUXPRODSUBSEMAM, AUXRECSUBSEMAM
			$this->service->executeOperationCiclo('AUXPRODSUBSEMAM',$row);
			$this->service->executeOperationCiclo('AUXRECSUBSEMAM',$row);

			//sugrupos principais - AUXSUBGRUPRINCIPALMENU
			$this->service->executeOperationCiclo('AUXSUBGRUPRINCIPALMENU',$row);

			parent::delete($request, $response);

			//deletar imagem
			if($row['OLD_FGFOTOSUBSEMAM'] != NULL && !empty($row['OLD_FGFOTOSUBSEMAM'])){
				$this->service->tryDeleteImage($row);
			}

			$this->connection->commit();
		} catch(\Exception $e){
			$response->setError(new Error($e->getMessage(),500));
			$this->connection->rollback();
		}
	}

		public function getOpcoesVariacaoReceita($cdreceita, $cdfilial){
		$query = "SELECT NROPCOES
					FROM VARIACAOREC
					WHERE CDRECEITA   = :CDRECEITA
					AND NRVARIACAOREC =
					  (SELECT MAX(NRVARIACAOREC)
					  FROM FILIRECEITA
					  WHERE CDFILIAL = :CDFILIAL
					  AND CDRECEITA  = :CDRECEITA
					  )";
		$param = array(
			"CDRECEITA" => $cdreceita,
			"CDFILIAL" => $cdfilial
		);
		$stmtMP = $this->connection->prepare($query);
		$stmtMP->execute($param);
		$result = $stmtMP->fetchAll(\PDO::FETCH_ASSOC);
		return $result;
	}
	public function getPreferenciasVariacaoReceita($cdreceita, $cdfilial){
		$query = "SELECT COUNT(SR.IDPREFERENCIA) AS IDPREFERENCIA
						FROM SUBRECEITAM SR
						JOIN RECEITAM RE
						ON SR.CDSUBRECEITA   = RE.CDRECEITA
						WHERE SR.NRORG       = :NRORG
						AND SR.CDRECEITA     = :CDRECEITA
						AND SR.NRVARIACAOREC =
						  (SELECT MAX(NRVARIACAOREC)
						  FROM FILIRECEITA
						  WHERE CDFILIAL = :CDFILIAL
						  AND CDRECEITA  = :CDRECEITA
						  )
						AND SR.IDPREFERENCIA = 'S'
						UNION ALL
						SELECT COUNT(PR.IDPREFERENCIA) AS IDPREFERENCIA
						FROM PRORECEITAM PR
						JOIN PRODUTO PD
						ON PR.CDPRODUTO      = PD.CDPRODUTO
						WHERE PR.NRORG       = :NRORG
						AND PR.CDRECEITA     = :CDRECEITA
						AND PR.NRVARIACAOREC =
						  (SELECT MAX(NRVARIACAOREC)
						  FROM FILIRECEITA
						  WHERE CDFILIAL = :CDFILIAL
						  AND CDRECEITA  = :CDRECEITA
						  )
						AND PR.IDPREFERENCIA = 'S'";
		$param = array(
			"CDRECEITA" => $cdreceita,
			"CDFILIAL" => $cdfilial,
			"NRORG" => $this->crudService->getEnvironment()->getNrOrg()
		);
		$stmtMP = $this->connection->prepare($query);
		$stmtMP->execute($param);
		$result = $stmtMP->fetchAll(\PDO::FETCH_ASSOC);
		if(empty($result)){
			return 0;
		} else {
			return array_reduce($result, function($i, $obj){
		    	return $i += $obj['IDPREFERENCIA'];
			});
		}
	}

	public function verificaFlex($cdreceita, $cdfilial, $response) {
		$query = "SELECT COUNT(IDRECFLEXIVEL) AS IDRECFLEXIVEL
						FROM RECEITAM
						WHERE CDRECEITA    = :CDRECEITA
						AND IDRECFLEXIVEL IN ('M', 'P')";
		$param = array(
			"CDRECEITA" => $cdreceita
		);
		$stmtMP = $this->connection->prepare($query);
		$stmtMP->execute($param);
		$result = $stmtMP->fetchAll(\PDO::FETCH_ASSOC);
		if(!empty($result)){
			if($result[0]['IDRECFLEXIVEL'] > 0){
				//Receita é flex
				$qtdOpcoes = $this->getOpcoesVariacaoReceita($cdreceita, $cdfilial);
				if(!empty($qtdOpcoes) && $qtdOpcoes[0]['NROPCOES'] > 0 ){
					$qtdPreferencias = $this->getPreferenciasVariacaoReceita($cdreceita, $cdfilial);
					if($qtdPreferencias !== $qtdOpcoes[0]['NROPCOES']){
						$this->setErrorMessage($cdreceita, " quantidade de opções igual a quantidade de preferencias selecionadas", $response);			
						return -1;
					}
				} else {
					$this->setErrorMessage($cdreceita, " quantidade de opções maior que 1", $response);		
					return -1;
				}
			}
		}
		return 1;
	}
	public function setErrorMessage($cdreceita, $tipo, $response) {
		$query = "SELECT DSRECEITA
		          FROM RECEITAM
		          WHERE CDRECEITA = :CDRECEITA";
		$param = array(
			"CDRECEITA" => $cdreceita
		);
		$resultado = $this->connection->fetchAll($query, $param);
		if(!empty($resultado) && $resultado[0]['DSRECEITA']) {
			$arrayError = "Receita ".$resultado[0]['DSRECEITA']." não possui ".$tipo.".";
			return $response->setError(new Error($arrayError, 500));
		}
	}

	public function saveComposicao($request, $response){
		$rows = $request->getDataSet()->getRows();
		$validate = 1;
		if (sizeof($rows)) {
			try{
				$this->connection->beginTransaction();
				$dataProduto = array();
				$dataReceita = array();
	
				foreach($rows as $row){
					if($row['ID'] == 'P'){
						array_push($dataProduto, $row);
					} else {
						array_push($dataReceita, $row);
					}
				}
				if(!empty($dataReceita)){
					$cdreceita = $dataReceita[0]['CDRECEITA']; 
					$cdfilial = $dataReceita[0]['CDFILIAL']; 
					$validate = $this->verificaFlex($cdreceita, $cdfilial, $response);
					
				}
				if($validate === 1){
					$this->saveDataComposicao('auxprodsubsemam', $dataProduto);
					$this->saveDataComposicao('auxrecsubsemam', $dataReceita);
					$this->verifyNrordemComposicao($rows[0]);
				}
				$this->connection->commit();
			} catch(\Exception $e){
				$response->setError(new Error($e->getMessage(), 500));
				$this->connection->rollback();
			}
		}
	}

	private function saveDataComposicao($dataSourceName, $rows){
		$newRequest = $newRequest = new DTO\Request\DataSet(new DataSet($dataSourceName, $rows), '','','');
		$rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
		$this->crudService->save($dataSourceName, $rowsToSave);
	}

	private function verifyNrordemComposicao($row){
		$conditions = array(
			array(
				'columnName' => 'CDFILIAL',
				'operator' => '=',
				'value' => $row['CDFILIAL']
			),
			array(
				'columnName' => 'CDCIA',
				'operator' => '=',
				'value' => $row['CDCIA']
			),
			array(
				'columnName' => 'CDMANCIA',
				'operator' => '=',
				'value' => $row['CDMANCIA']
			),
			array(
				'columnName' => 'CDCLASSE',
				'operator' => '=',
				'value' => $row['CDCLASSE']
			),
			array(
				'columnName' => 'NRSERVICOCIAFILI',
				'operator' => '=',
				'value' => $row['NRSERVICOCIAFILI']
			),
			array(
				'columnName' => 'CDSUBGRU',
				'operator' => '=',
				'value' => $row['CDSUBGRU']
			),
			array(
				'columnName' => 'NRSEQSUBGRUSEMAM',
				'operator' => '=',
				'value' => $row['NRSEQSUBGRUSEMAM']
			),
			array(
				'columnName' => 'CDCICLO',
				'operator' => '=',
				'value' => $row['CDCICLO']
			)
		);

		$filterCriteria = new FilterCriteria('coc00022_verify_nrordem_auxcomposicao', $conditions, null, null);
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array());
		$rows = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();

		if(!empty($rows) ){
			throw new \Exception('Operação bloqueada. Número de ordem já cadastrado!');
		}
	}

	public function saveItensCia($request, $response){
		$rows = $request->getDataSet()->getRows();
		$this->dataSourceNameToSave = 'auxprociasubsemam';

		try{
			$this->connection->beginTransaction();
			parent::save($request, $response);
			$this->verifyRepeatNrordem($rows);
			$this->connection->commit();
		} catch (\Exception $e){
			$response->setError(new Error($e->getMessage(), 500));
			$this->connection->rollback();
		}
	}

	private function verifyRepeatNrordem($rows){
		$conditions = array(
			array(
				'columnName' => 'CDFILIAL',
				'operator' => '=',
				'value' => $rows[0]['CDFILIAL']
			),
			array(
				'columnName' => 'CDCIA',
				'operator' => '=',
				'value' => $rows[0]['CDCIA']
			),
			array(
				'columnName' => 'CDMANCIA',
				'operator' => '=',
				'value' => $rows[0]['CDMANCIA']
			),
			array(
				'columnName' => 'NRSERVICOCIAFILI',
				'operator' => '=',
				'value' => $rows[0]['NRSERVICOCIAFILI']
			),
			array(
				'columnName' => 'CDCLASSE',
				'operator' => '=',
				'value' => $rows[0]['CDCLASSE']
			),
			array(
				'columnName' => 'CDSUBGRU',
				'operator' => '=',
				'value' => $rows[0]['CDSUBGRU']
			),
			array(
				'columnName' => 'NRSEQSUBGRUSEMAM',
				'operator' => '=',
				'value' => $rows[0]['NRSEQSUBGRUSEMAM']
			)
		);

		$filterCriteria = new FilterCriteria('coc00022_verify_nrordem_auxprociasubsemam', $conditions, null, null);
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array());
		$rowsProdcia = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();

		if(!empty($rowsProdcia) ){
			throw new \Exception('Operação bloqueada. Número de ordem já cadastrado!');
		}
	}

	public function duplicateSubgrupo($request, $response){
		$row = (array)$request->getRow();
		try{
			$this->connection->beginTransaction();
			$this->service->executeDuplicateProcedure($row);
			$this->connection->commit();
		} catch(\Exception $e){
			$response->setError(new Error($e->getMessage(), 500));
			$this->connection->rollback();
		}
	}

	public function effect($request, $response){
		$row = (array)$request->getRow();
		try{
			$this->connection->beginTransaction();
			if($row['USA_WF']){
				$rowWF = $this->mountRowToWorflow($row);
				$rowWF['DSMOTIVOACAO'] = $row['DSMOTIVOACAO'];
				$this->service->executeWorkflow($rowWF);
			} else {
				$this->service->executeEfetivaSubgrupo($row);
			}
			$this->connection->commit();
		} catch(\Exception $e){
			$response->setError(new Error($e->getMessage(),500));
			$this->connection->rollback();
		}
	}

	private function mountRowToWorflow($row){
		$primaryKeys = array(
			'CDFILIAL' => $row['CDFILIAL'],
			'CDCIA' => $row['CDCIA'],
			'CDMANCIA' => $row['CDMANCIA'],
			'NRSERVICOCIAFILI' => $row['NRSERVICOCIAFILI'],
			'CDCLASSE' => $row['CDCLASSE'],
			'CDSUBGRU' => $row['CDSUBGRU'],
			'NRSEQSUBGRUSEMAM' => $row['NRSEQSUBGRUSEMAM'],
			'NRORG' => $this->crudService->getEnvironment()->getNrOrg()
		);
		$nmitem = 'Unidade: '.$row['NMFILIAL']."\nCompanhia: ".$row['NMCIA']."\nMenu Grid: ".$row['DSMANUAL']
		."\nServiço: ".$row['SERVICO_CLASSE']."\nSubgrupo: ".$row['DSSUBGRU']."\nVigência: ".$row['DTINIVGSUMA'].' - '.$row['DTFIMVGSUMA'];
		return array(
			'PK' => $primaryKeys,
			'NMITEM' => $nmitem
		);
	}
}