<?php

namespace Teknisa\COC\Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;
use \Teknisa\GER\Services\CrudService;

class ControledePrecificacao extends \Teknisa\GER\Controllers\TeknisaCrud{

	protected $precificacaoItemfatService;
	protected $dataSourceManager;
	protected $environment;
	protected $parameterBagColumns = array();
	protected $messages;
	protected $tipoLoadingMapping = array(
		"2" => "V",
		"3" => "T",
		"4" => "%",
		"5" => "S"
	);

	function __construct(CrudService $crudService, ParameterBag $parameterBag, $dataSourceManager, $precificacaoItemfatService, $environment){
		$this->dataSourceManager = $dataSourceManager;
		$this->precificacaoItemfatService = $precificacaoItemfatService;
		$this->environment = $environment;
		parent::__construct($crudService, $parameterBag);
	}

	public function exportItemFat (DTO\Request $request, DTO\Response $response){
		$row = $request->getDataSet()->getRows()[0];
		$row['nrorg'] = $this->environment->getNrOrg();

		//Recebe a quantidade maxima de passageiros para o equipamento
		$qtd_max = $this->precificacaoItemfatService->findMaxEquipcia($row['CDCIA'], $row['CDFILIAL'], $this->environment->getNrOrg());

		$data_query = $this->precificacaoItemfatService->findItensFat($row['CDFILIAL'], $row['nrorg'], $row['CDCIA'], $row['CDMANCIA'], $row['DTINIVG'], $row['DTFIMVG'], $row['NRSERVICOCIAFILI'], $row['CDCLASSE'], $row['CDCICLO'], $qtd_max);
		$this->precificacaoItemfatService->createColumnsPax($qtd_max);
		$columns_name = $this->precificacaoItemfatService->getColumnsName();
		$listaErros= array();
		$erros = array();
		$newData = $this->calculaQuantidadePaxforLoading($data_query, $qtd_max);
		foreach ($newData as $row) {
			foreach ($row as $key => $value) {
			    if (is_null($value)) {
			    	if($key == "Preco" || $key == "Loading"){
			        	$row[$key] = "";
			    	} else if (($key>=1 && $key<=$qtd_max) && $row["Tipo de Loading"] == "3") {
			    		$nmsubgr = $row["Sub-Grupo"];
			    		$nmitem = $row["Item de Faturamento"];
			    		$erros[$nmsubgr."|".$nmitem] = true;
			    	}
			    }
			}
		}
		if(!empty($erros)) {
			foreach($erros as $key => $erro) {
				$nmsubgr = explode("|", $key)[0];
				$nmitem = explode("|", $key)[1];
				$erro = "O Subgrupo ".$nmsubgr.", do Item de Faturamento ".$nmitem.", não possui proporção variável definida.";
				array_push($listaErros, $erro);
			}
			$response->setError(new Error($listaErros, 500));
		}
	}

	public function downloadExportItemFat (DTO\Request $request, DTO\Response $response){
		$qtd_max = $this->precificacaoItemfatService->findMaxEquipcia($_REQUEST['cdcia'], $_REQUEST['cdfilial'], $this->environment->getNrOrg());
		if($_REQUEST['cdmancia'] === 'T'){
			$allCdmancia = $this->findAllCdmancia($_REQUEST['cdfilial'], $_REQUEST['cdcia']);
		}else{
			$allCdmancia = $_REQUEST['cdmancia'];
		}
		$data_query = $this->precificacaoItemfatService->findItensFat($_REQUEST['cdfilial'], $_REQUEST['nrorg'], $_REQUEST['cdcia'], $allCdmancia, $_REQUEST['dtinivg'], $_REQUEST['dtfimvg'], $_REQUEST['nrservicociafili'], $_REQUEST['cdclasse'], $_REQUEST['cdciclo'],  $qtd_max);
		$this->precificacaoItemfatService->createColumnsPax($qtd_max);
		$columns_name = $this->precificacaoItemfatService->getColumnsName();
		$listaErros= array();
		$erros = array();
		$outFile = null;
		$outFile .= $columns_name;
		$outFile = substr($outFile, 0, -1)."\n";
		$newData = $this->calculaQuantidadePaxforLoading($data_query, $qtd_max);

		foreach ($newData as $row) {
			foreach ($row as $key => $value) {
			    if (is_null($value)) {
			    	if($key == "Preco" || $key == "Loading"){
			        	$row[$key] = "";
			    	} else if (($key>=1 && $key<=$qtd_max) && $row["Tipo de Loading"] == "3") {
			    		$nmsubgr = $row["Sub-Grupo"];
			    		$nmitem = $row["Item de Faturamento"];
			    		$erros[$nmsubgr."|".$nmitem] = true;
			    	}
			    }
			    
			    if (in_array($key, array("CDFILIAL", "CDCIA", "CDMANCIA", "CDCICLO", "CDCLASSE", "CDSUBGRU")) ) {
					$row[$key] = '#'.$row[$key];
				}
			    
			    if($key == 'Tipo de Loading'){
			    	if(\Zeedhi\Framework\Util\Functions::arrayKeyExists($row[$key], $this->tipoLoadingMapping)){
			    		$row[$key] = $this->tipoLoadingMapping[$row[$key]];
			    	}else{
			    		$row[$key] = null;
			    	}
			    }
			}
			$outFile .= implode(';', $row)."\n";
		}
		$this->download_csv($_REQUEST['filename'], $outFile);
	}
	
	private function findAllCdmancia($cdfilialFromRequest, $cdciaFromRequest){
		$filtro = array(
            array(
			    "columnName"=> "CDFILIAL",
			    "operator"=> "=",
			    "value"=> $cdfilialFromRequest
            ),
            array(
			    "columnName"=> "CDCIA",
			    "operator"=> "=",
			    "value"=> $cdciaFromRequest
            ),
            array(
			    "columnName"=> "IDSITUAPROPCIA",
			    "operator"=> "IN",
			    "value"=> array("P", "A")
            )
		);
		
		$filterCriteriaManualciam = new FilterCriteria('manualciam', $filtro);
		$newFilterCriteriaManualciam = $this->crudService->populateParameterBag($filterCriteriaManualciam, array());
		$manuaisEmDesenvolvimentoAprovado = $this->dataSourceManager->findBy($newFilterCriteriaManualciam);
		$dataSetManuais = $manuaisEmDesenvolvimentoAprovado->getRows();
		$arrayManual = array();
		for ($i = 0; $i < sizeof($dataSetManuais); $i++) {
			array_push($arrayManual, $dataSetManuais[$i]['CDMANCIA']);
		}
		return implode("_",$arrayManual);
	}

	public function download_csv($filename, $outFile) {
		$fileContent = "";
		header('Accept-Ranges: none');
		header('Content-Description: File Transfer');
		header('Content-Type: application/octet-stream');
		header('Content-Transfer-Encoding: binary');
		header('Connection: Keep-Alive');
		header('Expires: 0');
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		header('Pragma: public');
		header("Content-Disposition: attachment; filename=".$filename.".csv");
		echo $outFile;
		exit;
	}

	public function calculaQuantidadePaxforLoading($data, $qtd_max){
		$newData = array();
		$value_by_tabela = null;
		$value = 0;
		foreach ($data as $row) {
			for($i = 1; $i<=$qtd_max; $i++){
				switch ($row["Tipo de Loading"]) { //Proporção
					case '4':
						//Percentual
						$row[$i] = (int) ceil(($row["Loading"] * $i) / 100);
						break;
					case '2':
						//Proporção fixa
						$row[$i] = (int) ceil($i/$row["Loading"]);
						break;
					case '5':
						//Valor unico
						$row[$i] = (int) $row["Loading"];
						break;
					case '3':
						//Calcula valor para o subgrupo de acordo com a tabela PROPSUBGRUSE
						$qtd_in_table = $this->precificacaoItemfatService->findPrecoTabelaProp($row['CDFILIAL'], $row['CDCIA'], $row['CDMANCIA'], $row['NRSERVICOCIAFILI'], $row['CDSUBGRU'], $row['NRORG']);
						foreach ($qtd_in_table as $interval) {
							$value = 0;
							if($i >= $interval['QTPAXDE'] && $i<=$interval['QTPAXATE']){
								$value = (int) $interval['QTSUBGRUPO'];
								$row[$i] = $value;
							} else {
								if(is_null($row[$i]))
									$row[$i] = $value;
							}
						}
						break;
				}
			}
			unset($row['NRSERVICO']);
			array_push($newData, $row);
		}
		return $newData;
	}
}