<?php

namespace Teknisa\COC\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use \Teknisa\GER\Utilities\MessagesProcedureInflightor;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\DBAL\Connection;

class DeleteMenuGrid extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName;
    protected $parameterBagColumns = array();
    protected $ExclusaoMenuGridProcedures;
    protected $ExclusaoServicoProcedures;
    protected $ExclusaoSubgrupoProcedures;
    protected $ExclusaoCicloProcedures;
    protected $MessagesProcedureInflightor;
    protected $dataSourceManager;

    public function __construct($crudService, $parameterBag, $environment, $dataSourceManager,
        $ExclusaoMenuGridProcedures, $ExclusaoServicoProcedures, $ExclusaoSubgrupoProcedures,
        $ExclusaoCicloProcedures, $MessagesProcedureInflightor) {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->ExclusaoMenuGridProcedures  = $ExclusaoMenuGridProcedures;
        $this->ExclusaoServicoProcedures   = $ExclusaoServicoProcedures;
        $this->ExclusaoSubgrupoProcedures  = $ExclusaoSubgrupoProcedures;
        $this->ExclusaoCicloProcedures     = $ExclusaoCicloProcedures;
        $this->MessagesProcedureInflightor = $MessagesProcedureInflightor;
        $this->dataSourceManager = $dataSourceManager;
    }

    public function executeDeleteMenuGrid(DTO\Request\DataSet $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows();
        $row = $row[0];
        $row['NRORG'] = $this->environment->getNrOrgTrab();
        try {
            $this->ExclusaoMenuGridProcedures->deleteMenuGrid($row['CDFILIAL'], $row['CDCIA'], $row['CDMANCIA'], $row['NRORG']);
        } catch (\Exception $e) {
            $response->setError(new Error($this->MessagesProcedureInflightor->convertMessage($e->getMessage()), 500));
        }
    }

    public function executeDeleteServico(DTO\Request\DataSet $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows();
        $row = $row[0];
        $row['NRORG'] = $this->environment->getNrOrgTrab();
        try {
            $this->ExclusaoServicoProcedures->deleteServico($row['CDFILIAL'], $row['CDCIA'], $row['CDMANCIA'], $row['NRSERVICOCIAFILI'], $row['CDCLASSE'], $row['NRORG']);
        } catch (\Exception $e) {
            $response->setError(new Error($this->MessagesProcedureInflightor->convertMessage($e->getMessage()), 500));
        }
    }

    public function executeDeleteSubgrupo(DTO\Request\DataSet $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows();
        $row = $row[0];
        $row['NRORG'] = $this->environment->getNrOrgTrab();
        try {
            $this->ExclusaoSubgrupoProcedures->deleteSubgrupo($row['CDFILIAL'], $row['CDCIA'], $row['CDMANCIA'], $row['NRSERVICOCIAFILI'], $row['CDCLASSE'], $row['CDSUBGRU'], $row['NRSEQSUBGRUSEMAM'], $row['NRORG']);
        } catch (\Exception $e) {
            $response->setError(new Error($this->MessagesProcedureInflightor->convertMessage($e->getMessage()), 500));
        }
    }

    public function executeDeleteCiclo(DTO\Request\DataSet $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows();
        $row = $row[0];
        $row['NRORG'] = $this->environment->getNrOrgTrab();
        try {
            $this->ExclusaoCicloProcedures->deleteCiclo($row['CDFILIAL'], $row['CDCIA'], $row['CDMANCIA'], $row['NRSERVICOCIAFILI'], $row['CDCLASSE'], $row['CDSUBGRU'], $row['NRSEQSUBGRUSEMAM'], $row['CDCICLO'], $row['NRORG']);
        } catch (\Exception $e) {
            $response->setError(new Error($this->MessagesProcedureInflightor->convertMessage($e->getMessage()), 500));
        }
    }


    public function multipleDeleteCiclo($request, $response){
        $row = (array)$request->getRow();
        try {
            $ciclos = $this->getCiclosFromDelete($row);
            foreach($ciclos as $ciclo){
                $this->ExclusaoCicloProcedures->deleteCiclo($ciclo['CDFILIAL'], $ciclo['CDCIA'], $ciclo['CDMANCIA'], $ciclo['NRSERVICOCIAFILI'], $ciclo['CDCLASSE'], $ciclo['CDSUBGRU'], $ciclo['NRSEQSUBGRUSEMAM'], $ciclo['CDCICLO'], $this->environment->getNrOrg());
            }
        } catch(\Exception $e){
            $response->setError(new Error($this->MessagesProcedureInflightor->convertMessage($e->getMessage()), 500));
        }
    }

    private function getCiclosFromDelete($row){
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL']
            ),
            array(
                'columnName' => 'CDCIA',
                'operator' => '=',
                'value' => $row['CDCIA']
            ),
            array(
                'columnName' => 'CDMANCIA',
                'operator' => '=',
                'value' => $row['CDMANCIA']
            ),
            array(
                'columnName' => 'NRSERVICOCIAFILI',
                'operator' => '=',
                'value' => $row['NRSERVICOCIAFILI']
            ),
            array(
                'columnName' => 'CDCLASSE',
                'operator' => '=',
                'value' => $row['CDCLASSE']
            ),
            array(
                'columnName' => 'SUBGRUPO',
                'operator' => '=',
                'value' => $row['SUBGRUPO']
            ),
            array(
                'columnName' => 'CICLO',
                'operator' => '=',
                'value' => $row['CICLO']
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $this->crudService->getEnvironment()->getNrOrg()
            )
        );

        $filterCriteria = new FilterCriteria('coc03600_get_ciclosubgrusemam_from_delete', $conditions, null, null, null);
        $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array());
        return $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
    }
}