<?php

namespace Teknisa\COC\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class FrequenciaCicloServico extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName;
	protected $parameterBagColumns;
	protected $connection;
	protected $dataSourceManager;

    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager, $connection) {
        $this->crudService = $crudService;
        $this->parameterBag=$parameterBag;
        $this->environment=$environment;
        $this->connection = $connection;
        $this->dataSourceManager = $dataSourceManager;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria(); 
        $this->dataSourceName = 'frcicloserm_servicociafili_classe_ciclo';
        $filterCriteria->addCondition('NRORG', "=", $this->environment->getNrOrg());
        $this->parameterBagColumns = array("CDFILIAL","CDCIA","NRVOO","CDTRECHO","NRORG");
        parent::find($request, $response);
    }
    
    public function findServicesToAdd(DTO\Request\Filter $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria(); 
        $this->dataSourceName = 'padvootrem_servicociafili_ciclo_classe_not_in_frcicloserm';
        $filterCriteria->addCondition('NRORG', "=", $this->environment->getNrOrg());
        $this->parameterBagColumns = array("CDFILIAL","CDCIA","NRVOO","CDTRECHO","NRORG");
        parent::find($request, $response);
    }
    
    public function importarFrequencia(DTO\Request $request, DTO\Response $response){
    	$dataSource = 'frcicloserm';
    	try{
    	    $this->connection->beginTransaction();
	    	$row = $request->getRow();
	    	
	    	$rowDestino = $this->montaRowDestino($row);
	    	
			$frequenciaNoPeriodo = $this->buscaFrequenciasNoPeriodo($row);
			
			$rowsASeremSalvas = $this->preparaRows($frequenciaNoPeriodo, $rowDestino);
			$this->apagaFrequenciaNoPeriodo($rowDestino);
			$newDataSet = new DataSet($dataSource, $rowsASeremSalvas);
			$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
			$rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSource);
			//rotina para verificar e vincular ciclos que nao existem no serviço destino.
			$this->vinculaCiclosServico($rowsToSave);
			$dataSet = new DataSet($dataSource, $rowsToSave);
			$this->dataSourceManager->persist($dataSet);
			$this->connection->commit();
    	} catch(\Exception $e){
    	    $this->connection->rollback();
    		$response->setError(new Error($e->getMessage(), 500));
    	}
		
    }
    
    public function exportarFrequencia(DTO\Request $request, DTO\Response $response){
    	$dataSource = 'frcicloserm';
    	try{
    	    $this->connection->beginTransaction();
			$row = $request->getRow();
            
            foreach($row['DATASETDESTINO'] as $destino){
                
                $this->atribuiVooTrechoServicoDestino($row['DATASETORIGEM'], $destino);
				$this->apagaFrequenciaNoPeriodoExp($destino);
				$this->salvaFrequenciaExportadas($row['DATASETORIGEM'], $destino);
                
            }
            $this->connection->commit();
		} catch(\Exception $e){
		    $this->connection->rollback();
    		$response->setError(new Error($e->getMessage(), 500));
    	}
    }
    
    public function montaRowDestino($row){
        
        return array(
                "CDFILIAL"=> $row['CDFILIAL'],
                "CDCIA" => $row['CDCIADESTINO'],
                "NRVOO" => $row['NRVOODESTINO'],
                "CDTRECHO" => $row['CDTRECHODESTINO'],
                "NRSERVICOCIAFILI" => $row['NRSERVICOCIAFILIDESTINO'],
                "CDCLASSE" => $row['CDCLASSEDESTINO'],
                "DTINICIAL" => $row['DTINICIAL'],
                "DTFINAL" => $row['DTFINAL']
            );
            
    }
    
    private function buscaFrequenciasNoPeriodo($row){
    	$query = 'SELECT CDFILIAL,
                         CDCIA,
                         NRVOO,
                         CDTRECHO,
                         NRSERVICOCIAFILI,
                         CDCLASSE,
                         CDCICLO,
                         DTCICLOSER,
                         NRORG
                    FROM FRCICLOSERM
                   WHERE CDFILIAL         = :CDFILIAL
                     AND CDCIA            = :CDCIA
                     AND NRVOO            = :NRVOO
                     AND CDTRECHO         = :CDTRECHO
                     AND NRSERVICOCIAFILI = :NRSERVICOCIAFILI 
                     AND CDCLASSE         = :CDCLASSE 
                     AND DTCICLOSER       BETWEEN :DTINICIAL AND :DTFINAL
                     AND NRORG = :NRORG';
                     
		$stmt = $this->connection->prepare($query);
		
		$params = array(
			'CDFILIAL'         => $row['CDFILIAL'],
			'CDCIA'            => $row['CDCIA'],
			'NRVOO'            => $row['NRVOO'],
			'CDTRECHO'         => $row['CDTRECHO'],
			'NRSERVICOCIAFILI' => $row['NRSERVICOCIAFILI'],
			'CDCLASSE'         => $row['CDCLASSE'],
			'DTINICIAL'        => $row['DTINICIAL'],
			'DTFINAL'          => $row['DTFINAL'],
			'NRORG'            => $this->environment->getNrOrgTrab()
		);
		
		$types = array(
		    'NRSERVICOCIAFILI' => \Doctrine\DBAL\Connection::PARAM_INT_ARRAY,
		    'CDCLASSE' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY
        );

		$stmt->execute($params, $types);
		
		$result = $stmt->fetchAll(\PDO::FETCH_ASSOC);

		return $result;
    	
    }
    
    private function apagaFrequenciaNoPeriodo($rowDestino){
    	
    	
    	$query = 'DELETE FROM FRCICLOSERM
                   WHERE CDFILIAL = :CDFILIAL
                     AND CDCIA = :CDCIA
                     AND NRVOO = :NRVOO
                     AND CDTRECHO = :CDTRECHO
                     AND NRSERVICOCIAFILI = :NRSERVICOCIAFILI
                     AND CDCLASSE = :CDCLASSE
                     AND DTCICLOSER BETWEEN :DTINICIAL AND :DTFINAL
                     AND NRORG = :NRORG';
                     
		$stmt = $this->connection->prepare($query);
		
		$params = array(
			'CDFILIAL'         => $rowDestino['CDFILIAL'],
			'CDCIA'            => $rowDestino['CDCIA'],
			'NRVOO'            => $rowDestino['NRVOO'],
			'CDTRECHO'         => $rowDestino['CDTRECHO'],
			'NRSERVICOCIAFILI' => $rowDestino['NRSERVICOCIAFILI'],
			'CDCLASSE'         => $rowDestino['CDCLASSE'],
			'DTINICIAL'        => $rowDestino['DTINICIAL'],
			'DTFINAL'          => $rowDestino['DTFINAL'],
			'NRORG'            => $this->environment->getNrOrgTrab()
		);
		
		$stmt->execute($params);
		
    }
    
    private function atribuiVooTrechoServicoDestino($origem, &$destino){
		$destino['CHAVE'] = $origem['CHAVE'];
    }
    
    private function preparaRows($rowsImportadas, $rowDestino){ 
    	
		foreach ($rowsImportadas as &$item) {
			$item['CDCIA']            = $rowDestino['CDCIA'];
			$item['NRVOO']            = $rowDestino['NRVOO'];
			$item['CDTRECHO']         = $rowDestino['CDTRECHO'];
			$item['NRSERVICOCIAFILI'] = $rowDestino['NRSERVICOCIAFILI'];
			$item['CDCLASSE']         = $rowDestino['CDCLASSE'];
			$item['changed']          = false;
			$item['__is_new']         = true;
		}
		
		return $rowsImportadas;
	
    }
    
    private function apagaFrequenciaNoPeriodoExp($rowDestino){
      
    	//@TODO - VERIFICAR A POSSIBILIDADE DE PASSAR ARRAY COMO PARAMETRO.
    	
    	$query = "DELETE FROM FRCICLOSERM
                   WHERE CDFILIAL         = :CDFILIAL
                     AND CDCIA            = :CDCIA
                     AND NRVOO            = :NRVOO
                     AND CDTRECHO         = :CDTRECHO
                     AND NRSERVICOCIAFILI||'_'||CDCLASSE IN ('".implode("','", $rowDestino['CHAVE'])."') 
                     AND DTCICLOSER       BETWEEN :DTINICIAL AND :DTFINAL
                     AND NRORG = :NRORG";
                     
	    $stmt = $this->connection->prepare($query);
	    
	    $params = array(
	      'CDFILIAL'         => $rowDestino['CDFILIAL'],
	      'CDCIA'            => $rowDestino['CDCIA'],
	      'NRVOO'            => $rowDestino['NRVOO'],
	      'CDTRECHO'         => $rowDestino['CDTRECHO'],
	      'DTINICIAL'        => $rowDestino['DTINICIAL'],
	      'DTFINAL'          => $rowDestino['DTFINAL'],
	      'NRORG'            => $this->environment->getNrOrgTrab()
	    );
	    
	    $stmt->execute($params);
    
    }
    
    private function salvaFrequenciaExportadas($rowOrigem, $rowDestino){
    	
    	//@TODO - VERIFICAR A POSSIBILIDADE DE PASSAR ARRAY COMO PARAMETRO.
    	$query = "INSERT INTO FRCICLOSERM (CDFILIAL, CDCIA, NRVOO, CDTRECHO, 
    	                                  NRSERVICOCIAFILI, CDCLASSE, CDCICLO, 
    	                                  DTCICLOSER, NRORG, DTINCLUSAO, 
    	                                  CDOPERINCLUSAO, DTULTATU, NRORGINCLUSAO, 
    	                                  NRORGULTATU, CDOPERULTATU, IDATIVO)
                  SELECT CDFILIAL, 
                         CDCIA, 
                         :NRVOO, 
                         :CDTRECHO, 
                         NRSERVICOCIAFILI, 
                         CDCLASSE, 
                         CDCICLO, 
                         DTCICLOSER, 
                         :NRORG,
                         SYSDATE, 
                         :CDOPER, 
                         SYSDATE, 
                         :NRORG, 
                         :NRORG, 
                         :CDOPER, 
                         'S'
                    FROM FRCICLOSERM
                   WHERE CDFILIAL         = :CDFILIAL
                     AND CDCIA            = :CDCIA
                     AND NRVOO            = :ORINRVOO
                     AND CDTRECHO         = :ORICDTRECHO
                     AND NRSERVICOCIAFILI||'_'||CDCLASSE IN ('".implode("','", $rowDestino['CHAVE'])."') 
                     AND DTCICLOSER       BETWEEN :DTINICIAL AND :DTFINAL
                     AND NRORG = :NRORG";
                 
	    $stmt = $this->connection->prepare($query);
	    
	    $params = array(
	      'CDFILIAL'         => $rowDestino['CDFILIAL'],
	      'CDCIA'            => $rowDestino['CDCIA'],
	      'NRVOO'            => $rowDestino['NRVOO'],
	      'ORINRVOO'         => $rowOrigem['NRVOO'],
	      'CDTRECHO'         => $rowDestino['CDTRECHO'],
	      'ORICDTRECHO'      => $rowOrigem['CDTRECHO'],
	      'DTINICIAL'        => $rowDestino['DTINICIAL'],
	      'DTFINAL'          => $rowDestino['DTFINAL'],
	      'CDOPER'           => $this->environment->getCdoperador(),
	      'NRORG'            => $this->environment->getNrOrgTrab()
	    );
	    
	    $stmt->execute($params);
    	
    }
    
    private function buscaCiclosNaoVinculados($rowsASeremSalvas){
    	
    	$ciclosNaoVinculados = array();
    	
    	foreach($rowsASeremSalvas as $dia){
    		
			$query = 'SELECT CDFILIAL,
                             CDCIA,
                             NRSERVICOCIAFILI
                             CDCLASSE,
                             CDCICLO,
                             NRORG
                        FROM SERVCICLO
                       WHERE CDFILIAL = :CDFILIAL
                         AND CDCIA = :CDCIA
                         AND NRSERVICOCIAFILI = :NRSERVICOCIAFILI
                         AND CDCLASSE = :CDCLASSE
                         AND CDCICLO = :CDCICLO
                         AND NRORG = :NRORG';
                         
			$stmt = $this->connection->prepare($query);
			
			$params = array(
				'CDFILIAL'         => $dia['CDFILIAL'],
				'CDCIA'            => $dia['CDCIA'],
				'NRSERVICOCIAFILI' => $dia['NRSERVICOCIAFILI'],
				'CDCLASSE'         => $dia['CDCLASSE'],
				'CDCICLO'          => $dia['CDCICLO'],
				'NRORG'            => $dia['NRORG']
			);
			
			$stmt->execute($params);
			
			$result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
    		if(empty($result)){
    			array_push($ciclosNaoVinculados, array(
					'CDFILIAL'         => $dia['CDFILIAL'],
                    'CDCIA'            => $dia['CDCIA'],
                    'NRSERVICOCIAFILI' => $dia['NRSERVICOCIAFILI'],
                    'CDCLASSE'         => $dia['CDCLASSE'],
                    'CDCICLO'          => $dia['CDCICLO'],
                    'NRORG'            => $dia['NRORG'],
					'__is_new'  => true
    			));
    		}
    	}
    	
		return array_unique($ciclosNaoVinculados, SORT_REGULAR);
		
    }
    
    private function vinculaCiclosServico($rowsToSave){
        
        $ciclosNaoVinculados = array();
        $ciclosNaoVinculados = $this->buscaCiclosNaoVinculados($rowsToSave);
        $this->vinculaCiclos($ciclosNaoVinculados);
        
    }
    
    private function vinculaCiclos($rows){
        
        $newDataSet = new DataSet('servciclo', $rows);
		$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
		$rowsToSave = $this->crudService->getRowsToSave($newRequest, 'servciclo');
		$dataSet = new DataSet('servciclo', $rowsToSave);
		$this->dataSourceManager->persist($dataSet);
        
    }
    
}