<?php

namespace Teknisa\COC\Controllers;

use Teknisa\GER\Utilities\Environment;
use Teknisa\GER\Services\CrudService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\Remote\FileUploader\Server;

class ImportacaoCustoController {

    protected $environment;
    protected $entityManager;
    protected $crudService;

    public function __construct($environment, $entityManager, $crudService) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->crudService = $crudService;
    }
    
    private function verifyDuplicateRegister($dataProd, $ownCdCia){
        $array = Array();
        $duplicate = Array();
        foreach($dataProd as $arr){
            if($ownCdCia){
        	    $key = $arr["CDPRODUTO"]."-".$arr["CDFILIAL"]."-".$arr["DTHRIMPCUSTO"]."-".$arr["CDCIA"];
            }else{
                $key = $arr["CDPRODUTO"]."-".$arr["CDFILIAL"]."-".$arr["DTHRIMPCUSTO"];
            }
        	if(\Zeedhi\Framework\Util\Functions::arrayKeyExists($key,$array)){
        		if(\Zeedhi\Framework\Util\Functions::arrayKeyExists($key,$duplicate)){
        			$duplicate[$key]["numberDuplicate"]++;
        		}else{
        			$duplicate[$key] = $arr;
        			$duplicate[$key]["numberDuplicate"] = 1;
        		}
        	}else{
        		$array[$key] = $arr;
	        }
        }
    	$msg = "COC26001_PRODUTOS_DUP_ARQ";
    	ksort($duplicate);
    	$produtos = "";
    	foreach($duplicate as $produto){
    	    $prod = "";
    	    if(empty($produto["DSPRODUTO"])){
    	        $prod .= $produto["CDARVPROD"];
    	    }else{
    	        $prod .= $produto["CDARVPROD"]." - ".$produto["DSPRODUTO"];
    	    }
    	    if(strlen($produtos) > 0)
    	        $produtos .= "#_#";
    	    $produtos .= $prod . "@@@" . $produto["numberDuplicate"];
    	}
    	if($produtos != ""){
    	    throw new \Exception(nl2br($msg."#_#".$produtos));
    	}
    }
    
    public function uploadCsvFile(DTO\Request $request, DTO\Response $response){
        try {
            $file = $request->getDataSet()->getRows()[0];
            $id = $request->getDataSet()->getRows()[1]['ID'];
            $b64 = explode(",", $file['b64File'])[1];

            $fileStr = base64_decode($b64);
            $fileStr = str_replace('"', '', $fileStr);
            $fileArray = explode("\n", $fileStr);
            //remove as linhas de cabeçalho que nao serao uteis
            array_shift($fileArray);

            $dataProdInvalido = Array();

            $dataProdAP      = Array();
            $dataProdAPCia   = Array();
            $selectProdAp = "SELECT CDPRODUTO FROM PRODUTOAP WHERE (CDFILIAL = :CDFILIAL) AND (CDPRODUTO = :CDPRODUTO) AND (DTCUSTOAP = :DTCUSTOAP)";
            $selectProdApCia = "SELECT CDPRODUTO FROM PRODUTOAPCIA WHERE (CDFILIAL = :CDFILIAL) AND (CDCIA = :CDCIA) AND (CDPRODUTO = :CDPRODUTO) AND (DTCUSTOAP = :DTCUSTOAP)";
            $currentLine = 1;
            $prodNotFound = Array();
            foreach($fileArray as $dataLine){
                $currentLine++;
                $lineArray = explode(";", $dataLine);
                //recebe o CDPRODUTO a partir do CDARVPROD e verifica se ele existe
                $produto = $this->entityManager->getConnection()->executeQuery(
                    "SELECT CDPRODUTO FROM PRODUTO WHERE CDARVPROD = '".$lineArray[0]."'")->fetch();
                
                if ($produto["CDPRODUTO"]){
                    $lineArray[9] = $produto["CDPRODUTO"];

                    //aplica zero fill na CDFILIAL
                    $lineArray[2] = str_pad($lineArray[2], 4, "0", STR_PAD_LEFT);
                    $dataCustoAp = explode("/", $lineArray[8]);
                    if(!is_array($dataCustoAp) || count($dataCustoAp) != 3 || !checkdate($dataCustoAp[1],$dataCustoAp[0],$dataCustoAp[2])){
                        throw new \Exception("COC26001_DATA_INVALIDAS");
                    }
                    if ($lineArray[3]){
                        //recebe o CDCIA a partir do SGCIA e substitui o mesmo na rowsCiafili
                        $cia = $this->entityManager->getConnection()->executeQuery(
                            "SELECT CDCIA FROM COMPANHIA WHERE SGCIA ='".$lineArray[3]."'")->fetch();
                        if($cia["CDCIA"] == null){
                            throw new \Exception("COC26001_CIA_INVALIDAS");
                        }
                        $lineArray[3] = $cia["CDCIA"];
                        
                        //verifica e cria vinculo na CIAFILI para a filial e cia informadas
                        $paramsCiafili = array(
                            "CDFILIAL" => $lineArray[2],
                            "CDCIA" => $lineArray[3]
                        );
                        $queryCiafili = "SELECT CDCIA FROM CIAFILI WHERE CDFILIAL = :CDFILIAL
                        AND CDCIA = :CDCIA";
                        if (empty($this->entityManager->getConnection()->executeQuery($queryCiafili, $paramsCiafili)->fetch())){
                            $ciafiliToSave = array(array(
                                "CDFILIAL" => $lineArray[2],
                                "CDCIA" => $lineArray[3],
                                "__is_new" => true
                            ));
                            $dataSetCiafili = new DataSet("ciafili", $ciafiliToSave);
                            $requestCiafili = new DTO\Request\DataSet($dataSetCiafili, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                            $rowsCiafili    = $this->crudService->getRowsToSave($requestCiafili, "ciafili");
                            $this->crudService->save("ciafili", $rowsCiafili);
                        }

                        //verifica se o registro ja existe
                        $params = array(
                            "CDFILIAL" => $lineArray[2],
                            "CDCIA" => $lineArray[3],
                            "CDPRODUTO" => $lineArray[9],
                            "DTCUSTOAP" => $lineArray[8]
                        );
                        $isNew = empty($this->entityManager->getConnection()->executeQuery($selectProdApCia, $params)->fetch());
                        if ($id == 'S' || $isNew) {
                            //cria o registro a ser inserido/atualizado
                            array_push($dataProdAPCia, array(
                                "CDARVPROD" => $lineArray[0],
                                "DSPRODUTO" => $lineArray[1],
                                "CDFILIAL" => $lineArray[2],
                                "CDCIA" => $lineArray[3],
                                "VRMEDLIQAP" => str_replace(',', '.', $lineArray[4]),
                                "VRMEDBRUAP" => str_replace(',', '.', $lineArray[5]),
                                "VRULTCLIQAP" => str_replace(',', '.', $lineArray[6]),
                                "VRULTCBRUAP" => str_replace(',', '.', $lineArray[7]),
                                "DTCUSTOAP" => $lineArray[8],
                                "CDPRODUTO" => $lineArray[9],
                                "DTHRIMPCUSTO" => $this->environment->getCurrentTime()->format("d/m/Y H:i:s"),
                                "CDOPIMPCUSTO" => $this->environment->getCdOperador(),
                                "__is_new"   => $isNew
                            ));
                        }
                    } else {
                        //verifica se o registro ja existe
                        $params = array(
                            "CDFILIAL" => $lineArray[2],
                            "CDPRODUTO" => $lineArray[9],
                            "DTCUSTOAP" => $lineArray[8]
                        );
                        $isNew = empty($this->entityManager->getConnection()->executeQuery($selectProdAp, $params)->fetch());
                        
                        if ($id == 'S' || $isNew) {
                            //cria o registro a ser inserido/atualizado
                            array_push($dataProdAP, array(
                                "CDARVPROD" => $lineArray[0],
                                "DSPRODUTO" => $lineArray[1],
                                "CDFILIAL" => $lineArray[2],
                                "VRMEDLIQAP" => str_replace(',', '.', $lineArray[4]),
                                "VRMEDBRUAP" => str_replace(',', '.', $lineArray[5]),
                                "VRULTCLIQAP" => str_replace(',', '.', $lineArray[6]),
                                "VRULTCBRUAP" => str_replace(',', '.', $lineArray[7]),
                                "DTCUSTOAP" => $lineArray[8],
                                "CDPRODUTO" => $lineArray[9],
                                "DTHRIMPCUSTO" => $this->environment->getCurrentTime()->format("d/m/Y H:i:s"),
                                "CDOPIMPCUSTO" => $this->environment->getCdOperador(),
                                "__is_new"   => $isNew
                            ));
                            
                        }
                    }
                } else {
                    if(!empty($lineArray[0])){
                        array_push($prodNotFound, $currentLine);    
                    }
                    if(!empty($lineArray[1])) {
                        array_push($dataProdInvalido, $lineArray[0].' - '.$lineArray[1]);
                    }
                }
            };
            
            if(count($prodNotFound)){
                $msg = "COC26001_PROD_N_ENCONTRADOS";
                $msg .= '#_#'.join(", ",$prodNotFound);
                throw new \Exception(nl2br($msg));
            }
            if (count($dataProdAPCia)){
                $this->verifyDuplicateRegister($dataProdAPCia, true);
                $dataSetProdApCia = new DataSet("produtoapcia", $dataProdAPCia);
                $requestProdApCia = new DTO\Request\DataSet($dataSetProdApCia, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                $rowsApCia        = $this->crudService->getRowsToSave($requestProdApCia, "produtoapcia");
                $this->crudService->save("produtoapcia", $rowsApCia);
            }

            if (count($dataProdAP)){
                $this->verifyDuplicateRegister($dataProdAP, false);
                $dataSetProdAp = new DataSet("produtoap", $dataProdAP);
                $requestProdAp = new DTO\Request\DataSet($dataSetProdAp, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                $rowsAp        = $this->crudService->getRowsToSave($requestProdAp, "produtoap");
                $this->crudService->save("produtoap", $rowsAp);
            }

            if (count($dataProdInvalido)){
                $response->addDataSet(new DataSet('errorMessage', $dataProdInvalido));
            }
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
        }
    }
}