<?php

namespace Teknisa\COC\Controllers;

use Teknisa\GER\Utilities\Environment;
use Teknisa\GER\Services\CrudService;
use Teknisa\COC\Services\ImportarItensFaturamentoService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\Remote\FileUploader\Server;

class ImportarItensFaturamento {

    protected $tabela;
    protected $arquivo;
    protected $importService;
    protected $listaErrosPlanoVoo = array();
    protected $vrindice = 0.0;
    protected $tipoLoadingMapping = array(
        "V" => "2",
        "T" => "3",
        "%" => "4",
        "S" => "5"
    );
    protected $dataSourceManager;

    const FILE_SEPARATOR = ";";
    const PROPORCAO_HUM_POR_PAX = "1";
    const PROPORCAO_VARIAVEL = "2";
    const PROPORCAO_TABELA = "3";
    const PROPORCAO_LOADING = "4";
    const PROPORCAO_QUANTIDADE_FIXA = "5";
    const ID_TABELA_PROPORCIONAL = "S";
    const ID_UTILIZA_LOADING = "S";

    public function __construct($environment, $entityManager, $crudService, $importService, $dataSourceManager) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->crudService = $crudService;
        $this->importService = $importService;
        $this->dataSourceManager = $dataSourceManager;
    }
    
    function isEmpty($var) {
        if (empty($var) && $var !== '0') {
            return true;
        }else{
            return false;
        }
    }

    private function defineMoedaUtilizada($moeda, $precoReal, $precoDolar) {
        if ($this->isEmpty($precoReal) && $this->isEmpty($precoDolar)) {
            array_push($moeda,'erro_null');
        }elseif(!$this->isEmpty($precoReal) && !$this->isEmpty($precoDolar)){
            array_push($moeda,'erro');
        }elseif(!$this->isEmpty($precoReal)){
            if(floatval($precoReal) > 0){
                array_push($moeda, 'real');
            }
            else{
                array_push($moeda, 'erro_zero');
            }
        }elseif(!$this->isEmpty($precoDolar)){
            if(floatval($precoDolar) > 0){
                array_push($moeda, 'dolar');
            }
            else{
                array_push($moeda, 'erro_zero');
            }
        }
        return $moeda;
    }
    

    private function validaArquivo($arquivo) {

        //Valida a extensão padrão do arquivo
        $this->arquivo['EXTENSAO'] = pathinfo($arquivo['name'], PATHINFO_EXTENSION);

        if($this->arquivo['EXTENSAO'] !== 'csv')
            throw new \Exception("COC21000_DEVE_SER_CSV");

        //Extrai do arquivo o conteúdo principal e o cabeçalho
        $b64 = explode(",", $arquivo['b64File'])[1];
        $decode = base64_decode($b64);
        $decode = str_replace('"', '', $decode);
        $linhas = explode("\n", $decode);
        
        //Valida o separador padrão 
        $cabecalho = $linhas[0];

        if(!strpos($cabecalho, self::FILE_SEPARATOR))
            throw new \Exception("COC21000_SEPARADOR_VALOR_INVALIDO");

        $this->arquivo['CABECALHO'] = explode(self::FILE_SEPARATOR, $cabecalho);

        array_shift($linhas);
        
        $conteudoPrincipal = array();
        for($i=0;$i<count($linhas);$i++) {
            /*
                Adiciona apenas as linhas com conteúdo
                utiliza a função strcspn para procurar conteúdo após o separador padrão ";"
            */ 
            if(strcspn($linhas[$i],self::FILE_SEPARATOR))
                $conteudoPrincipal[] = explode(self::FILE_SEPARATOR, $linhas[$i]);
        }
        $this->arquivo['CONTEUDO_PRINCIPAL'] = $conteudoPrincipal;
        if(count($this->arquivo['CONTEUDO_PRINCIPAL']) == 0){
            throw new \Exception("COC21000_ARQUIVO_VAZIO");
        }
    }
    
    private function precoInformado($nroLinha){
        return ($this->arquivo['CONTEUDO_PRINCIPAL'][$nroLinha][$this->importService->getColumnsPreco()['REAL']] != 0 || $this->arquivo['CONTEUDO_PRINCIPAL'][$nroLinha][$this->importService->getColumnsPreco()['DOLAR']] != 0);
    }

    private function iniciaParametros($tabela, $sobrescrever) {

        $this->tabela['CDCIA'] = $tabela['CDCIA'];
        $this->tabela['CDFILIAL'] = $tabela['CDFILIAL'];
        $this->tabela['DTFIMVIGTAB'] = $tabela['DTFIMVIGTAB'];
        $this->tabela['DTINIVIGTAB'] = $tabela['DTINIVIGTAB'];
        $this->tabela['NRTABPRECO'] = $tabela['TABELA'];
        $this->tabela['SOBRESCREVER'] = $sobrescrever['ID'];
    
    }
    
    private function preencheListaPlanodeVoos($existePlanoVoo, $existe){
        $itemFat = "COC21000_IT_IMPORT_COM_PLANOVOO" ."#_#" . $existe[0]['DSSERVICOCIAFILI'] ."#_#" . $existe[0]['SGCLASSE'] ."#_#" . $existe[0]['IDCICLO'];
        array_push($this->listaErrosPlanoVoo, $itemFat);
        foreach ($existePlanoVoo as $planoVoo) {
            $voo = "COC21000_IT_IMPORT_VOO" ."#_#" . $planoVoo['NRVOO'] ."#_#" . substr($planoVoo['DTHRPLAVOOPV'], 0, 10);
            array_push($this->listaErrosPlanoVoo, $voo);
        }
    }

    private function exibeErrosNoConteudo(){
        $listaLinhaErrosPreco = $this->arquivo['LINHAS_ERRO']['PRECO_NAO_INFORMADO'];
        $listaErrosNoConteudo = array();
        for($i=0; $i<count($listaLinhaErrosPreco); $i++){
            $nroLinhaConteudo = $listaLinhaErrosPreco[$i];
            $nroLinhaArquivo = $nroLinhaConteudo + 2; //por causa do cabeçalho
            $linha = $this->arquivo['CONTEUDO_PRINCIPAL'][$nroLinhaConteudo];
            $erro = "COC21000_ERRO_CONTEUDO_ARQUIVO"."#_#".$linha[$this->importService->getColumnsSubGrupo()['NOME']]."#_#".$linha[$this->importService->getColumnsServico()['NOME']]."#_#". 
                    $linha[$this->importService->getColumnsClasse()['NOME']]."#_#".$linha[$this->importService->getColumnsCiclo()['NOME']] .
                    "#_#". $nroLinhaArquivo;
            array_push($listaErrosNoConteudo, $erro);
        }
        return $listaErrosNoConteudo;
    }
    
    
    private function validaFilialCompanhia(){
        //Valida se a filial e a companhia informadas no arquivo de importação estão de acordo com a Tabela de Preços
        $cdfilial = substr($this->arquivo['CONTEUDO_PRINCIPAL'][0][$this->importService->getColumnsUnidade()['CODIGO']], 1);
        $cdcia = substr($this->arquivo['CONTEUDO_PRINCIPAL'][0][$this->importService->getColumnsCompanhia()['CODIGO']], 1);
        if($cdfilial != $this->tabela['CDFILIAL'] || $cdcia != $this->tabela['CDCIA'])
            throw new \Exception("COC21000_ARQ_N_PERTENCE_CIA", 1);
    }
    
    public function uploadCsvItemFat(DTO\Request $request, DTO\Response $response) {

        try {
            //Validações iniciais do arquivo de importação
            $rows = $request->getDataSet()->getRows();
            $this->iniciaParametros($rows[2], $rows[1]);
            $this->validaArquivo(current($rows));
            $this->validaFilialCompanhia();

            //Extrai do conteúdo principal do arquivo os parametros iniciais
            $nrservicociafilipreco = $this->arquivo['CONTEUDO_PRINCIPAL'][0][$this->importService->getColumnsServico()['CODIGO']]; 
            $nrservicociafili = $this->arquivo['CONTEUDO_PRINCIPAL'][0][$this->importService->getColumnsServico()['CODIGOPRECO']];
            $cdclasse = substr($this->arquivo['CONTEUDO_PRINCIPAL'][0][$this->importService->getColumnsClasse()['CODIGO']], 1);
            $cdciclo = substr($this->arquivo['CONTEUDO_PRINCIPAL'][0][$this->importService->getColumnsCiclo()['CODIGO']], 1);
            $cdmancia = substr($this->arquivo['CONTEUDO_PRINCIPAL'][0][$this->importService->getColumnsMenuGrid()['CODIGO']], 1);
            $tabelaProporcional = $this->arquivo['CONTEUDO_PRINCIPAL'][0][$this->importService->getColumnsTabelaProp()['UTILIZA']];
            $utilizaLoading = $this->arquivo['CONTEUDO_PRINCIPAL'][0][$this->importService->getColumnsLoading()['UTILIZA']];
            $servico = array(
                "NRSERVICOCIAFILIPRECO" => $nrservicociafilipreco,
                "NRSERVICOCIAFILI" => $nrservicociafili,
                "CDCLASSE" => $cdclasse,
                "CDCICLO" => $cdciclo,
                "CDMANCIA" => $cdmancia,
                "TABELA_PROPORCIONAL" => $tabelaProporcional,
                "UTILIZA_LOADING"       => $utilizaLoading
            );
            

            $this->arquivo['SERVICOS'] = array();
            $this->arquivo['LINHAS_ERRO'] = array();
            $this->arquivo['LINHAS_ERRO']['PRECO_NAO_INFORMADO'] = array();
            $size = sizeof($this->arquivo['CONTEUDO_PRINCIPAL']);

            for($i=0; $i < $size; $i++) {
                $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsUnidade()['CODIGO']] = substr($this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsUnidade()['CODIGO']], 1);
                $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsCompanhia()['CODIGO']] = substr($this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsCompanhia()['CODIGO']], 1);
                $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsMenuGrid()['CODIGO']] = substr($this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsMenuGrid()['CODIGO']], 1);
                $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsCiclo()['CODIGO']] = substr($this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsCiclo()['CODIGO']], 1);
                $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsClasse()['CODIGO']] = substr($this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsClasse()['CODIGO']], 1);
                $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsLoading()['TIPO']] = \Zeedhi\Framework\Util\Functions::arrayKeyExists($this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsLoading()['TIPO']], $this->tipoLoadingMapping) ? $this->tipoLoadingMapping[$this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsLoading()['TIPO']]] : null;
                $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsPreco()['REAL']] = floatval(str_replace(',', '.', $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsPreco()['REAL']]));
                $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsPreco()['DOLAR']] = floatval(str_replace(',', '.', $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsPreco()['DOLAR']]));
                
                //Sinaliza erro na linha com Preço não informado
                if(!$this->precoInformado($i))
                    $this->arquivo['LINHAS_ERRO']['PRECO_NAO_INFORMADO'][] = $i;
                
                if($cdclasse !== $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsClasse()['CODIGO']] ||
                   $cdciclo !== $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsCiclo()['CODIGO']] ||
                   $nrservicociafili !== $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsServico()['CODIGOPRECO']]) {
                   
                   //Inclui item de faturamento no array de serviços       
                   array_push($this->arquivo['SERVICOS'], $servico);
                   
                   //Extrai os parametros para o novo Item de Faturamento
                   $nrservicociafilipreco = $this->arquivo['CONTEUDO_PRINCIPAL'][0][$this->importService->getColumnsServico()['CODIGO']]; 
                   $nrservicociafili = $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsServico()['CODIGOPRECO']];
                   $cdclasse = $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsClasse()['CODIGO']];
                   $cdciclo = $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsCiclo()['CODIGO']];
                   $cdmancia = $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsMenuGrid()['CODIGO']];
                   $tabelaProporcional = $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsTabelaProp()['UTILIZA']];
                   $utilizaLoading = $this->arquivo['CONTEUDO_PRINCIPAL'][$i][$this->importService->getColumnsLoading()['UTILIZA']];
                   $servico = array(
                        "NRSERVICOCIAFILIPRECO" => $nrservicociafilipreco,
                        "NRSERVICOCIAFILI"      => $nrservicociafili,
                        "CDCLASSE"              => $cdclasse,
                        "CDCICLO"               => $cdciclo,
                        "CDMANCIA"              => $cdmancia,
                        "TABELA_PROPORCIONAL"   => $tabelaProporcional,
                        "UTILIZA_LOADING"       => $utilizaLoading
                    );
                }
            }
            //Inclui item de faturamento no array de serviços       
            array_push($this->arquivo['SERVICOS'], $servico);
            
            //Se existir erro, exibe a lista para o usuário
            if($this->exibeErrosNoConteudo()){
                $response->setError(new Error($this->exibeErrosNoConteudo(), 800));
            } else {
                $this->listaErrosPlanoVoo = array();
                $this->calculaPrecoPax();
                //Valida se existem Plano de Voos já gerados para os Itens de Faturamento exibindo-os na listagem de erros
                if(!empty($this->listaErrosPlanoVoo))
                    $response->setError(new Error($this->listaErrosPlanoVoo, 800));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
   private function same($arr) {
      return $arr === array_filter($arr, function ($element) use ($arr) {
            return ($element === $arr[0]);
        }); 
   } 
    
   private function calculaPrecoPax() {

        $this->vrindice = $this->getVrindice();
        
        for($key=0; $key < count($this->arquivo['SERVICOS']); $key++){
            
            $filtro = array(
               "CDCLASSE" => $this->arquivo['SERVICOS'][$key]['CDCLASSE'],
               "NRSERVICOCIAFILI" => $this->arquivo['SERVICOS'][$key]['NRSERVICOCIAFILI'],
               "CDCICLO" => $this->arquivo['SERVICOS'][$key]['CDCICLO'],
               "CDMANCIA" => $this->arquivo['SERVICOS'][$key]['CDMANCIA']
            );
            $subgrupos = array_filter($this->arquivo['CONTEUDO_PRINCIPAL'], function ($element) use ($filtro) { 
               return ($element[$this->importService->getColumnsClasse()['CODIGO']] == $filtro['CDCLASSE'] && 
                       $element[$this->importService->getColumnsServico()['CODIGOPRECO']] == $filtro['NRSERVICOCIAFILI'] && 
                       $element[$this->importService->getColumnsCiclo()['CODIGO']] == $filtro['CDCICLO'] &&
                       $element[$this->importService->getColumnsMenuGrid()['CODIGO']] == $filtro['CDMANCIA']); 
            });

            $precoUnitReal = 0.0;
            $precoUnitDolar = 0.0;
            $precoUnitLoadingReal = 0.0;
            $precoUnitLoadingDolar = 0.0;
            $somaPrecoUnitDolar = 0.0;
            $somaPrecoUnitReal = 0.0;
            $somaPrecoUnitLoadingDolar = 0.0;
            $somaPrecoUnitLoadingReal = 0.0;
            
            $geraTabelaProporcao = false;
            $usaPrecoComLoading = false;
            
            $idprop = '';
            $qtprop = 0;
            $idpropsubsemam = array();
            $qtpropsubsemam = array();
            
            $preco_total = array();
            $preco_pax = array();

            foreach($subgrupos as $subgrupo){
                
                $qtpropsubsemam[] = $subgrupo[$this->importService->getColumnsLoading()['VALOR']];
                $idpropsubsemam[] = $subgrupo[$this->importService->getColumnsLoading()['TIPO']];

                $idprop = $subgrupo[$this->importService->getColumnsLoading()['TIPO']];
                $qtprop = $subgrupo[$this->importService->getColumnsLoading()['VALOR']];

                $precoUnitReal = $subgrupo[$this->importService->getColumnsPreco()['REAL']];
                $precoUnitDolar = $subgrupo[$this->importService->getColumnsPreco()['DOLAR']];
                switch ($idprop) {
                    case '4':
                        $precoUnitLoadingReal = round($precoUnitReal * ($qtprop/100), 2);
                        $precoUnitLoadingDolar = round($precoUnitDolar * ($qtprop/100), 2);
                        break;
                    case '5':
                        $precoUnitLoadingReal = round($precoUnitReal * $qtprop, 2);
                        $precoUnitLoadingDolar = round($precoUnitDolar * $qtprop, 2);
                        break;
                    default:
                        $precoUnitLoadingReal = $precoUnitReal;
                        $precoUnitLoadingDolar = $precoUnitDolar;
                        break;
                }

                $somaPrecoUnitDolar += $precoUnitDolar;
                $somaPrecoUnitReal += $precoUnitReal;
                $somaPrecoUnitLoadingDolar += $precoUnitLoadingDolar;
                $somaPrecoUnitLoadingReal += $precoUnitLoadingReal;
                
            }    
            
            if(count($subgrupos) == 1) {
                
                $idprop = $idpropsubsemam[0];
                
            }elseif(count($subgrupos) > 1) {
                if(!$this->same($idpropsubsemam)) {
                    
                    $idprop = self::PROPORCAO_HUM_POR_PAX;
                    $geraTabelaProporcao = true;
                    
                }else{
                    if($idpropsubsemam[0] == self::PROPORCAO_VARIAVEL) {
                        if(!$this->same($qtpropsubsemam)) {
                            
                            $idprop = self::PROPORCAO_HUM_POR_PAX;
                            $geraTabelaProporcao = true;
                            
                        }else{
                            
                            $idprop = self::PROPORCAO_VARIAVEL;
                            
                        }
                    
                    }else if($idpropsubsemam[0] == self::PROPORCAO_TABELA) {
                    
                        $idprop = self::PROPORCAO_HUM_POR_PAX;
                        $geraTabelaProporcao = true;
                    
                    }else if($idpropsubsemam[0] == self::PROPORCAO_LOADING) {
                    
                        if(strtoupper($this->arquivo['SERVICOS'][$key]['UTILIZA_LOADING']) == self::ID_UTILIZA_LOADING && $this->same($qtpropsubsemam)){
                            $idprop = self::PROPORCAO_LOADING;
                            $usaPrecoComLoading = false;
                        } else {
                            $idprop =  self::PROPORCAO_HUM_POR_PAX;
                            $usaPrecoComLoading = true;
                        }
                        if(strtoupper($this->arquivo['SERVICOS'][$key]['TABELA_PROPORCIONAL']) == self::ID_TABELA_PROPORCIONAL)
                            $geraTabelaProporcao = true;

                    }else if($idpropsubsemam[0] == self::PROPORCAO_QUANTIDADE_FIXA) {
                    
                        $idprop = self::PROPORCAO_QUANTIDADE_FIXA;
                        
                        if(!$this->same($qtpropsubsemam)){
                            $usaPrecoComLoading = true;
                        }
                    }
                }
            }
            
            $this->arquivo['SERVICOS'][$key]['IDPROPORCAO'] = $idprop;
            $this->arquivo['SERVICOS'][$key]['QTPROPORCAO'] = null;
            $this->arquivo['SERVICOS'][$key]['VRPRESERV'] = $usaPrecoComLoading ? $somaPrecoUnitLoadingReal : $somaPrecoUnitReal;
            $this->arquivo['SERVICOS'][$key]['VRPRESERVD'] = $usaPrecoComLoading ? $somaPrecoUnitLoadingDolar : $somaPrecoUnitDolar;
            
            $this->corrigePreco($this->arquivo['SERVICOS'][$key]['VRPRESERV'], $this->arquivo['SERVICOS'][$key]['VRPRESERVD']);
            
            $nrseqpreco = $this->salvarItem($key);
            if($nrseqpreco){
                $this->importService->deleteProprecosem($this->tabela['CDFILIAL'], $this->tabela['CDCIA'], $this->tabela['NRTABPRECO'], $nrseqpreco, 
                                                      $this->arquivo['SERVICOS'][$key]['CDCLASSE'], 
                                                      $this->arquivo['SERVICOS'][$key]['CDCICLO'], 
                                                      $this->arquivo['SERVICOS'][$key]['NRSERVICOCIAFILI'], $this->environment->getNrOrg());
                /* 
                                        --- 09/08/2017 ---
                    Inclusão dos Subgrupos importados na tabela de SUBGRUTABPRECOM.
                    Grava o subgrupo com o preço aberto para cada Item de Faturamento incluído pela tabela.
                */
                    $this->importService->deleteSubgrupoTabelaPreco($this->tabela['CDFILIAL'], $this->tabela['CDCIA'], $this->tabela['NRTABPRECO'],
                                                      $this->arquivo['SERVICOS'][$key]['NRSERVICOCIAFILI'],
                                                      $this->arquivo['SERVICOS'][$key]['CDCLASSE'], 
                                                      $this->arquivo['SERVICOS'][$key]['CDCICLO'], 
                                                      $nrseqpreco,
                                                      $this->environment->getNrOrg());
                    $this->salvaSubgrupoTabelaPreco($nrseqpreco, $key, $subgrupos);

                if($geraTabelaProporcao){
                    $this->salvarProporcao($this->arquivo['SERVICOS'][$key], $nrseqpreco, $subgrupos);
                }
            }
        }
   }
   
   //criar metodo privado para o preenchimento das datas apos pesquisa
   
   private function findSubgrupoManual($subgrupo, $nrseqpreco){
        $filtro = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => str_pad($this->tabela['CDFILIAL'], 4, "0", STR_PAD_LEFT)
            ),   
            array(
                "columnName" => "CDCIA",
                "operator" => "=",
                "value" => str_pad($this->tabela['CDCIA'], 4, "0", STR_PAD_LEFT)
            ),  
            array(
                "columnName" => "NRSERVICOCIAFILI",
                "operator" => "=",
                "value" => $subgrupo[$this->importService->getColumnsServico()['CODIGO']]
            ),   
            array(
                "columnName" => "CDCLASSE",
                "operator" => "=",
                "value" => str_pad($subgrupo[$this->importService->getColumnsClasse()['CODIGO']], 4, "0", STR_PAD_LEFT)
            ),   
            array(
                "columnName" => "CDMANCIA",
                "operator" => "=",
                "value" => str_pad($subgrupo[$this->importService->getColumnsMenuGrid()['CODIGO']], 4, "0", STR_PAD_LEFT)
            ),   
            array(
                "columnName" => "NRSEQSUBGRUSEMAM",
                "operator" => "=",
                "value" => $subgrupo[$this->importService->getColumnsSubGrupo()['SEQUENCIAL']]
            ),   
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrgTrab()
            )   
        );
        
        $filterCriteria = new FilterCriteria('subgrusemam', $filtro);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        return $dataSet->getRows()[0];
        
   }
   
   private function salvaSubgrupoTabelaPreco($nrseqpreco, $index, $subgrupos){
        $data = array();
        foreach ($subgrupos as $subgrupo) {
            $subgrupoManual = $this->findSubgrupoManual($subgrupo, $nrseqpreco);
            
            $row = array(
                "CDFILIAL" =>                $this->tabela['CDFILIAL'],
                "CDCIA" =>                   $this->tabela['CDCIA'],
                "NRTABPRECO" =>              $this->tabela['NRTABPRECO'],
                "NRSERVICOCIAFILIPRECO" =>   $subgrupo[$this->importService->getColumnsServico()['CODIGOPRECO']],
                "CDCLASSE" =>                $subgrupo[$this->importService->getColumnsClasse()['CODIGO']],
                "CDCICLO" =>                 $subgrupo[$this->importService->getColumnsCiclo()['CODIGO']],
                "NRSEQPRECO" =>              intval($nrseqpreco),
                "NRSUBGRUTABPRECOM" =>       NULL,
                "__is_new" =>                true,
                "__is_new_code" =>           true,
                "__new_code_pk" =>           "SUBGRUTABPRECOM".$this->tabela['CDFILIAL'].$this->tabela['CDCIA'].$this->tabela['NRTABPRECO'].intval($nrseqpreco),
                "__table_pk" =>              "NRSUBGRUTABPRECOM",
                "__new_code_length" =>       5,
                "CDMANCIA" =>                $subgrupo[$this->importService->getColumnsMenuGrid()['CODIGO']],
                "NRSERVICOCIAFILI" =>        $subgrupo[$this->importService->getColumnsServico()['CODIGO']],
                "CDSUBGRU" =>                substr($subgrupo[$this->importService->getColumnsSubGrupo()['CODIGO']], 1),
                "NRSEQSUBGRUSEMAM" =>        $subgrupo[$this->importService->getColumnsSubGrupo()['SEQUENCIAL']],
                "CDPRODUTO" =>               NULL,        
                "CDRECEITA" =>               NULL,        
                "IDPROPSUBSEMAM" =>          $subgrupo[$this->importService->getColumnsLoading()['TIPO']],       
                "QTPROPSUBSEMAM" =>          $subgrupo[$this->importService->getColumnsLoading()['VALOR']],        
                "VRPRECOSUBGRU" =>           $subgrupo[$this->importService->getColumnsPreco()['REAL']],           
                "VRPRECOSUBGRUD" =>          $subgrupo[$this->importService->getColumnsPreco()['DOLAR']],           
                "DTINIVGSUMAINCLUSAO" =>     $subgrupoManual['DTINIVGSUMA'],
                "DTFIMVGSUMAINCLUSAO" =>     $subgrupoManual['DTFIMVGSUMA']
            );
            array_push($data, $row);
        }
        if(!empty($data))
            $this->importService->saveSubgrupoTabelaPreco($data);
   }

   private function salvarItem($nroServico){
        $servico = $this->arquivo['SERVICOS'][$nroServico];
        
        $existe = $this->findItensTabelaPreco($servico);
        $validations = $this->validaRegistro($existe, $this->tabela);

        $nrseqpreco = NULL;

        if ($validations['save']) {
            $this->importService->insertMenuGridTabelaPreco($this->tabela['CDFILIAL'], $this->tabela['CDCIA'], $this->tabela['NRTABPRECO'], $servico['CDMANCIA']);
            if(empty($existe)){
                $nrseqpreco = null;
            } else {
                if(!empty($existe[0])){
                    $nrseqpreco = $existe[0]['NRSEQPRECO'];
                } else {
                    $nrseqpreco = null;
                }
            }
            $row = array(
                "CDFILIAL" => $this->tabela['CDFILIAL'],
                "CDCIA" => $this->tabela['CDCIA'],
                "CDCICLO" => $servico['CDCICLO'],
                "CDCLASSE" => $servico['CDCLASSE'],
                "NRTABPRECO" => $this->tabela['NRTABPRECO'],
                "CDMANCIA" => $servico['CDMANCIA'],
                "NRORG" => $this->environment->getNrOrg(),
                "NRSEQPRECO" => $nrseqpreco,
                "NRSERVICOCIAFILI" => $servico['NRSERVICOCIAFILI'],
                "__is_new" => $validations['isnew'],
                "__is_new_code" => $validations['isnew'],
                "__new_code_pk" => "PRECOSECIAM".$this->tabela['CDFILIAL'].$this->tabela['CDCIA'].$this->tabela['NRTABPRECO'],
                "__table_pk" => "NRSEQPRECO",
                "__new_code_length" => 3,
                "__columns_to_return" => ["NRSEQPRECO"],
                "IDPROPORCAO" => $servico['IDPROPORCAO'],
                "QTPROPORCAO" => $servico['QTPROPORCAO'],
                "DTFIMVIGSE" => $this->tabela['DTFIMVIGTAB'],
                "DTINIVIGSE" => $this->tabela['DTINIVIGTAB'],
                "VRPRESERV" => $servico['VRPRESERV'],
                "VRPRESERVD" => $servico['VRPRESERVD'],
                "IDCALCTABPROP" => strtoupper($servico['TABELA_PROPORCIONAL']) == 'S' ?  strtoupper($servico['TABELA_PROPORCIONAL']) : 'N',
                "IDUTILLOADIGUAL" => strtoupper($servico['UTILIZA_LOADING'])   == 'S' ?  strtoupper($servico['UTILIZA_LOADING'])     : 'N'
            ); 
            $nrseqpreco = $this->importService->savePrecoseciam($row);
        }
        return $nrseqpreco;
   }
   
   private function corrigePreco(&$precoReal, &$precoDolar){
       if ($precoReal == 0 && $precoDolar > 0) {
           if($this->vrindice){
             $precoReal = $precoDolar * $this->vrindice;
           } else {
             $precoReal = 0.0;
           }
       } else if ($precoReal > 0 && $precoDolar == 0) {
           if($this->vrindice){
               $precoDolar = $precoReal / $this->vrindice;
           } else {
               $precoDolar = 0.0;
           }
       }
   }
   
   private function salvarProporcao($servico, $nrseqpreco, $subgrupos){

       $cdfilial = $this->tabela['CDFILIAL'];
       $cdcia = $this->tabela['CDCIA'];
       $cdciclo = $servico['CDCICLO'];
       $cdclasse = $servico['CDCLASSE'];
       $nrtabpreco = $this->tabela['NRTABPRECO'];
       $nrorg = $this->environment->getNrOrg();
       $nrservicociafili = $servico['NRSERVICOCIAFILI'];
       $precoUnitReal = 0.0;
       $precoUnitDolar = 0.0;
       $precoParcialReal = 0.0;
       $precoParcialDolar = 0.0;
       $precoReal = 0.0;
       $precoDolar = 0.0;
       $somaPrecoReal = 0.0;
       $somaPrecoDolar = 0.0;
       
       $qtdePax = 1;
       $qtdeItens = 1;
       
       for($i=$this->importService->getColumnsPax()['COLUNA']; $i < count($this->arquivo['CABECALHO']); $i++){
           $somaPrecoReal = 0.0;
           $somaPrecoDolar = 0.0;
           $qtdePax = intval($this->arquivo['CABECALHO'][$i]);
           foreach($subgrupos as $subgrupo){
               $qtdeItens = ($subgrupo[$i] == '-') ? 1 : $subgrupo[$i];
               $precoUnitReal = $subgrupo[$this->importService->getColumnsPreco()['REAL']];
               $precoUnitDolar = $subgrupo[$this->importService->getColumnsPreco()['DOLAR']];
               
               $precoParcialReal = $qtdeItens * $precoUnitReal;
               $precoParcialDolar = $qtdeItens * $precoUnitDolar;
               
               $somaPrecoReal += $precoParcialReal;
               $somaPrecoDolar += $precoParcialDolar;
           }
           $precoReal = round($somaPrecoReal/$qtdePax, 2);
           $precoDolar = round($somaPrecoDolar/$qtdePax, 2);
           $this->corrigePreco($precoReal, $precoDolar);
           
           
           
          $row = array (
                "CDFILIAL" => $cdfilial,
                "CDCIA" => $cdcia,
                "CDCICLO" => $cdciclo,
                "CDCLASSE" => $cdclasse,
                "NRTABPRECO" => $nrtabpreco,
                "NRORG" => $nrorg,
                "NRSERVICOCIAFILI" => $nrservicociafili,
                "__is_new" => true,
                "NRSEQPRECO" => $nrseqpreco,
                "QTPAXINICIAL" => $qtdePax,
                "QTPAXFINAL" => $qtdePax,
                "VRPRESERVPRO" => $precoReal,
                "VRPRESERVDPRO" => $precoDolar
            );
            $this->importService->insertPrecoProporcional($row);
       }
   }

   private function getVrindice(){
        $conditions = array(
            array (
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            )
        );
        try {
            $dataSet = $this->crudService->getFilteredDataSet("coc21000_movindice", $conditions);
            if($dataSet->getRows()) {
                return $dataSet->getRows()[0]['VRINDICE'];
            } else {
                return false;
            }
        } catch(\Exception $e){
             throw new \Exception($e->getMessage(), $e->getCode());
        }
    }


    private function findItensTabelaPreco ($servico) {
        //Find na tabela de Precoseciam para verificar se o Item de Faturamento ja existe
        $conditions = array(
            array (
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => str_pad($this->tabela['CDFILIAL'], 4, "0", STR_PAD_LEFT)
            ),
            array (
                "columnName" => "CDCIA",
                "operator" => "=",
                "value" => str_pad($this->tabela['CDCIA'], 4, "0", STR_PAD_LEFT)
            ),
            array (
                "columnName" => "CDCICLO",
                "operator" => "=",
                "value" => str_pad($servico['CDCICLO'], 4, "0", STR_PAD_LEFT)
            ),
            array (
                "columnName" => "CDCLASSE",
                "operator" => "=",
                "value" => str_pad($servico['CDCLASSE'], 4, "0", STR_PAD_LEFT)
            ),
            array (
                "columnName" => "NRTABPRECO",
                "operator" => "=",
                "value" => $this->tabela['NRTABPRECO']
            ),
            array (
                "columnName" => "NRSERVICOCIAFILI",
                "operator" => "=",
                "value" => $servico['NRSERVICOCIAFILI']
            ),
            array (
                "columnName" => "DTINIVIGSE",
                "operator" => "=",
                "value" => $this->tabela['DTINIVIGTAB']
            ),
            array (
                "columnName" => "DTFIMVIGSE",
                "operator" => "=",
                "value" => $this->tabela['DTFIMVIGTAB']
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet('precoseciam_between_vigencia', $conditions);
        if (empty($dataSet->getRows()))
            return [];
        else
            return $dataSet->getRows();
    }
    
    /*
        Valida inclusão de novos Itens de Faturamento
        se for pra Sobrescrever os Itens, valida se já existem Plano de Voos já gerados
        para a Vigência atual do Item, caso exista trava a inclusão, senão faz o update 
        ou inclui o novo Preço para o item.
        @return array validations
        Validations ->
                        save  = true 
                        isnew = false  -> Updata o Item de Faturamento já existente

                        save  = true
                        isnew = true  -> Cria o Item de Faturamento com novo Preço e Vigência

                        save  = false 
                        isnew = true/false -> Bloqueia a inclusão ou alteração do Item de Faturamento.
    */
    private function validaRegistro($existe, $tabela) {
        $validations = array(
            "save" => false,
            "isnew" => false
        );
        if($tabela['SOBRESCREVER'] == 'S') {
            if(!empty($existe)){
                $existePlanoVoo = $this->importService->verificaPlanoVoo($existe);
            }
            if (!empty($existePlanoVoo)) { //true
                $validations['save'] = false;
                $this->preencheListaPlanodeVoos($existePlanoVoo, $existe);                
            } else {
                $validations['save'] = true;
                if(!empty($existe)) {
                    $validations['isnew'] = false;
                } else{
                    $validations['isnew'] = true;
                } 
            }
        } else {
            if (empty($existe)) {
                $validations['save'] = true;
                $validations['isnew'] = true;
            } else {
                $validations['save'] = false;
            }
        }
        return $validations;
    }

}