<?php
namespace Teknisa\COC\Controllers;

use \Zeedhi\Framework\Report\ReportService;
use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DTO\Request\Row;

class WorkflowViewer extends \Zeedhi\Framework\Controller\Report {

    protected $environment;
    protected $crudService;
    protected $fileServerParams;
    protected $dataSourceManager;

    public function __construct(ReportService $reportService, $environment, $crudService, $dataSourceManager, $fileServerParams) {
        $this->fileServerParams = $fileServerParams;
        $this->crudService = $crudService;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        parent::__construct($reportService);
    }

    protected $reportMapping = array (
        'COC00004_VR' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"      =>"NRORG",
              "CDRECEITA"  =>"CDRECEITA",
              "NRWORKFLOW"  => "NRWORKFLOW",
              "CDOPERADOR"  => "CDOPERADOR"
            ),
            'reportName'
        ),
        'COC00004_VV' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"      =>"NRORG",
              "CDRECEITA"  =>"CDRECEITA",
              "NRVARIACAOREC"  =>"NRVARIACAOREC",
              "NRWORKFLOW"  => "NRWORKFLOW",
              "CDOPERADOR"  => "CDOPERADOR"
            ),
            'reportName'
        ),
        'COC00004_VC' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"      => "NRORG",
              "CDRECEITA"  => "CDRECEITA",
              "NRVARIACAOREC"  => "NRVARIACAOREC",
              "WFLOWITEM"  => "WFLOWITEM",
              "NRWORKFLOW"  => "NRWORKFLOW",
              "CDOPERADOR"  => "CDOPERADOR"
            ),
            'reportName'
        ),
        'COC00004_VIC' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"      => "NRORG",
              "CDRECEITA"  => "CDRECEITA",
              "NRVARIACAOREC"  => "NRVARIACAOREC",
              "NRWORKFLOW"  => "NRWORKFLOW",
              "CDOPERADOR"  => "CDOPERADOR"
            ),
            'reportName'
        ),
        'COC00004_VM' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"      => "NRORG",
              "CDRECEITA"  => "CDRECEITA",
              "NRVARIACAOREC"  => "NRVARIACAOREC",
              "NRWORKFLOW"  => "NRWORKFLOW",
              "CDOPERADOR"  => "CDOPERADOR"
            ),
            'reportName'
        ),
        'COC00004_PD' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
              "NRORG"      => "NRORG",
              "CDPRODUTO"  => "CDPRODUTO"
            ),
            'reportName'
        ),
        'COC03700' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'NRORG' => 'NRORG',
                'CDFILIAL' => 'CDFILIAL',
                'CDCIA' => 'CDCIA',
                'CDMANCIA' => 'CDMANCIA',
                'NRSERVICOCIAFILI' => 'NRSERVICOCIAFILI',
                'CDCLASSE' => 'CDCLASSE',
                'CDSUBGRUPO' => 'CDSUBGRUPO',
                'NRSEQSUBGRUSEMAM' => 'NRSEQSUBGRUSEMAM',
                'AUXNRSEQSUBGRUSEMAM' => 'AUXNRSEQSUBGRUSEMAM',
                'AUXCDSUBGRU' => 'AUXCDSUBGRU',
                'HIDEIMG' =>'HIDEIMG',
                'FILESERVPATH' => 'FILESERVPATH'
            )
        )
    );

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {

        $row = $request->getRow();
        $row['NRORG'] = $this->environment->getNrorgTrab();
        $dataSourceWorkflow = 'workflow';
        $dataSourceDadoswflow = 'dadoswflow';
        $parameterBagColumns = array();

        $filter = array(
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $this->environment->getNrorgTrab()
            ),
            array(
                'columnName' => 'NRWORKFLOW',
                'operator' => '=',
                'value' => $row['NRWORKFLOW']
            )
        );


        $filterCriteriaWorkflow = new FilterCriteria($dataSourceWorkflow, $filter);
        $filterCriteriaWorkflow =  $this->crudService->populateParameterBag($filterCriteriaWorkflow, $parameterBagColumns);
        $dataSetWorkflow = $this->dataSourceManager->findBy($filterCriteriaWorkflow);
        $registrosWorkflow = $dataSetWorkflow->getRows();

        foreach ($registrosWorkflow as $workflow) {
            $processo = $workflow['NRPROCESSOAPROVA'];
            $item = $workflow['NRITEMPROCAPROVA'];
        }



        $relMapping = array(
            1 => array(
                1    => 'COC00004_VR',
                2    => 'COC00004_VR',
                3    => 'COC00004_VR',
                4    => 'COC00004_VR',
                5    => 'COC00004_VR',
                6    => 'COC00004_VR',
                7    => 'COC00004_VR',
                8    => 'COC00004_VR',
                9    => 'COC00004_VR',
                10   => 'COC00004_VR',
                11   => 'COC00004_VR',
                12   => 'COC00004_VR',
                13   => 'COC00004_VR',
                14   => 'COC00004_VC',
                15   => 'COC00004_VM',
                16   => 'COC00004_VIC',
                17   => 'COC00004_VIC',
                18   => 'COC00004_VV',
                19   => 'COC00004_VV',
                20   => 'COC00004_VV',
                21   => 'COC00004_VV',
                22   => 'COC00004_VV',
                23   => 'COC00004_VV',
                24   => 'COC00004_VV'
            ),
            2 => array(
                1    => 'COC00004_PD'
            ),
            4 => array(
                24 => 'COC03700'
            )
        );

        $row['__report_name'] = $relMapping[$processo][$item];

        $filterCriteriaDadoswflow = new FilterCriteria($dataSourceDadoswflow, $filter);
        $filterCriteriaDadoswflow =  $this->crudService->populateParameterBag($filterCriteriaDadoswflow, $parameterBagColumns);
        $dataSetDadoswflow = $this->dataSourceManager->findBy($filterCriteriaDadoswflow);
        $registrosDadoswflow = $dataSetDadoswflow->getRows();

        foreach ($registrosDadoswflow as $dadosWflow) {
            $parameters = explode(';', $dadosWflow['DSPRIMARYKEY']);
            foreach ($parameters as $parameter) {
                if(strlen($parameter) != 0){
                    $keyAndValue = explode('=', $parameter);
                    $row[$keyAndValue[0]] = $keyAndValue[1];
                }
            }
        }

        if($row['__report_name'] == 'COC00004_VC'){
            $itemPks = array();
            $columnsToTransform = array(
                "CDPRODUTO",
                "CDSUBRECEITA"
            );
            foreach ($registrosDadoswflow as $dadosWflow) {
                $parameters = explode(';', $dadosWflow['DSPRIMARYKEY']);
                foreach ($parameters as $parameter) {
                    $keyAndValue = explode('=', $parameter);
                    if(in_array($keyAndValue[0], $columnsToTransform)){
                        $row['WFLOWITEM'] = $keyAndValue[1];
                    }
                }
            }
        } else if($row['__report_name'] == 'COC03700'){
            $row['AUXNRSEQSUBGRUSEMAM'] = $row['NRSEQSUBGRUSEMAM'];
            $row['AUXCDSUBGRU'] = $row['CDSUBGRU'];
            $row['CDSUBGRUPO'] = $row['CDSUBGRU'];
            $row['HIDEIMG'] = 'f';
            $row['FILESERVPATH'] = $this->fileServerParams->getFileserverPath();;

            $row['NRSEQSUBGRUSEMAM'] = $this->getNrseq($row);
        }
        $row['CDOPERADOR'] = $this->environment->getCdoperador();

    }

    private function buildParamsByColumn($column){
        $row;
        if (strpos($column, ';')) {
            $row[substr($column, 0, strpos($column, '='))] = substr($column, strpos($column, '=')+1, strpos($column, ';') - strlen($column));
            $this->buildParamsByColumn(substr($column, strpos($column, ';')+1));
        }else{
            $row[substr($column, 0, strpos($column, '='))] = substr($column, strpos($column, '=')+1);
        }

        return $row;
    }

    private function buildConditionByColumn($column){

        /*
            Monta o 'condition' do filter por meio de uma coluna.
             - ';' separador de pk/fk.
             - '=' separador de valores da pk/fk.
        */

        if (strpos($column, ';')) {

            // caso tenha pk e fk's

            array_push($this->condition,
                array(
                    'columnName' => substr($column, 0, strpos($column, '=')),
                    'operator' => '=',
                    'value' => substr($column, strpos($column, '=')+1, strpos($column, ';') - strlen($column))
                )
            );

            $this->buildConditionByColumn(substr($column, strpos($column, ';')+1));

        }else{

            // caso não tenha fk's

            array_push($this->condition,
                array(
                    'columnName' => substr($column, 0, strpos($column, '=')),
                    'operator' => '=',
                    'value' => substr($column, strpos($column, '=')+1)
                )
            );
        }

        return $this->condition;

    }

    public function getNrseq($row){
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL']
            ),
            array(
                'columnName' => 'CDCIA',
                'operator' => '=',
                'value' => $row['CDCIA']
            ),
            array(
                'columnName' => 'CDMANCIA',
                'operator' => '=',
                'value' => $row['CDMANCIA']
            ),
            array(
                'columnName' => 'NRSERVICOCIAFILI',
                'operator' => '=',
                'value' => $row['NRSERVICOCIAFILI']
            ),
            array(
                'columnName' => 'CDCLASSE',
                'operator' => '=',
                'value' => $row['CDCLASSE']
            ),
            array(
                'columnName' => 'CDSUBGRU',
                'operator' => '=',
                'value' => $row['CDSUBGRU']
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $row['NRORG']
            )
        );


        $filterCriteria =  new FilterCriteria('coc03600_get_nrseqsubgrusemam', $conditions);
        $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array());
        $rows = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
        return $rows[0]['NRSEQSUBGRUSEMAM'];
    }


}